;;======================================================================;;
;;			Digitlky	 					;;
;;======================================================================;;
;;									;;
;; Program:		Digitln hodiny s vyuzitim filament LEDs	;;
;; Code:		Jindra Ficik					;;
;; Platform:		Microchip PIC16F15323, 1 Mhz			;;
;; Date:		08.03.2022					;;
;; First release:	08.03.2022					;;
;; LastDate:		29.04.2022					;;
;;									;;
;;======================================================================;;

; Processor PIC 16F15323 running at 1MHz internal clock + 32768Hz external timer
;
; Revisions:
;
; 08/03/2022	Start of writing code.
; 29/04/2022	Update for filament LEDs

; ----- Definitions

#define		__VERNUM	.1
#define		__SUBVERNUM	.0
#define		__VERDAY	0x29
#define		__VERMONTH	0x04
#define		__VERYEAR	0x22


                list    p=16F15323,r=hex

		errorlevel -302
		;errorlevel -305
		;errorlevel -306

	        INCLUDE "P16F15323.INC"

; CONFIG1
; __config 0xFFEC
 __CONFIG _CONFIG1, _FEXTOSC_OFF & _RSTOSC_HFINT1 & _CLKOUTEN_OFF & _CSWEN_ON & _FCMEN_ON
; CONFIG2
; __config 0xF7FF
 __CONFIG _CONFIG2, _MCLRE_OFF & _PWRTE_OFF & _LPBOREN_OFF & _BOREN_ON & _BORV_LO & _ZCD_OFF & _PPS1WAY_OFF & _STVREN_ON
; CONFIG3
; __config 0xFF9F
 __CONFIG _CONFIG3, _WDTCPS_WDTCPS_31 & _WDTE_OFF & _WDTCWS_WDTCWS_7 & _WDTCCS_SC
; CONFIG4
; __config 0xFFEF
 __CONFIG _CONFIG4, _BBSIZE_BB512 & _BBEN_OFF & _SAFEN_ON & _WRTAPP_OFF & _WRTB_OFF & _WRTC_OFF & _WRTSAF_OFF & _LVP_OFF
; CONFIG5
; __config 0xFFFF
 __CONFIG _CONFIG5, _CP_OFF


; ----- Macros

#define		DNOP		goto	$+1

;#define PCBv2		; uncoment if V2 board is used

#define		RAMINI0		0x020		; 80 bytes
#define		RAMINI1		0x0A0		; 80 bytes
#define		RAMINI2		0x120		; 80 bytes
#define		RAMINT		0x070		; 16 bytes	available in all banks

; ----- Constant values

FXTAL		equ	D'1000000'		; oscillator frequency

;         +---U---+
;    Vdd -|Vdd Vss|- Vss
; Clk in -|RA5 RA0|- Parallel load   (HC595 / 12)
;        -|RA4 RA1|- Serial clock    (HC595 / 11)
; MCLR   -|RA3 RA2|- Serial data out (HC595 / 14)
; Key H+ -|RC5 RC0|- Key Show Sec
; Key H- -|RC4 RC1|- Key Sec 0
; Key M+ -|RC3 RC2|- Key M-
;         +-------+

; 595: Serial clock: SCK = Low to High _/- content of SER transmitted to stage 0, stage 0 -> 1 -> 2...
; 595: Parallel load: RCK = Low to High _/- content of shift register shifted into parallel register

; PMD = PERIPHERAL MODULE DISABLE
PMD0_INI	equ	b'01000101'	; CLKRMD CLKR enabled; SYSCMD SYSCLK enabled; FVRMD FVR disabled; IOCMD IOC disabled; NVMMD NVM disabled; 
PMD1_INI	equ	b'10000010'	; TMR0MD TMR0 enabled; TMR1MD TMR1 disabled; TMR2MD TMR2 enabled; NCOMD DDS(NCO) disabled; 
PMD2_INI	equ	b'01100111'	; ZCDMD ZCD disabled; CMP1MD CMP1 disabled; ADCMD ADC disabled; CMP2MD CMP2 disabled; DAC1MD DAC1 disabled; 
PMD3_INI	equ	b'00111111'	; CCP2MD CCP2 disabled; CCP1MD CCP1 disabled; PWM4MD PWM4 disabled; PWM3MD PWM3 disabled; PWM6MD PWM6 disabled; PWM5MD PWM5 disabled; 
PMD4_INI	equ	b'01010001'	; CWG1MD CWG1 disabled; MSSP1MD MSSP1 disabled; UART1MD EUSART disabled; 
PMD5_INI	equ	b'00011110'	; CLC3MD CLC3 disabled; CLC4MD CLC4 disabled; CLC1MD CLC1 disabled; CLC2MD CLC2 disabled; 

; PIN manager
LATA_INI    equ 0x00	    ; all outputs to zero
LATC_INI    equ 0x00	    ; all outputs to zero

TRISA_INI   equ b'00111000' ; RA0-RA2 output to 74595, RA3 input - mclr, RA4 not used, RA5 32768 Hz input
TRISC_INI   equ b'00111111' ; all input - buttons

ANSELA_INI  equ b'00000000' ; all digital
ANSELC_INI  equ b'00000000' ; all digital

WPUA_INI    equ	b'00011000'	; WPU for not used inputs
WPUC_INI    equ	b'00111111' ; for all buttons

ODCONA_INI  equ	0x00	    ; disable open drain outputs
ODCONC_INI  equ	0x00	    ; disable open drain outputs
   
; oscillator management
OSCCON1_INI equ 0x60	    ; NOSC HFINTOSC; NDIV 1:1; 
;OSCCON2 is read only
OSCCON3_INI equ 0x00	    ; CSWHOLD may proceed; 
OSCEN_INI   equ 0x00	    ; MFOEN disabled; LFOEN disabled; ADOEN disabled; EXTOEN disabled; HFOEN disabled; 
OSCFRQ_INI  equ 0x00	    ; HFFRQ 1_MHz; 
OSCSTAT_INI equ 0x00	    ; MFOR not ready; do not understand, it is read only
;OSCTUNE_INI equ 0x20	    ; HFTUN 32; do not understand why MCC set this?
OSCTUNE_INI equ 0x00	    ; HFTUN 0; default

; Timer2 management			; Timer 2 is used for 1 sec pulses
T2CLKCON_INI	equ 0x00    ; T2CS T2CKIPPS; 32768 Hz crystal used for 1 sec pulses
T2HLT_INI   equ 0x00    ; T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
T2RST_INI   equ 0x00	; T2RSEL T2INPPS pin; 
T2PR_INI    equ 0xFF	; T2PR 255 = 1 Hz; 
T2CON_INI   equ b'11110000'	; T2CKPS 1:128; T2OUTPS 1:1; TMR2ON on; 

T2INPPS_INI	equ 0x05	; RA5 is used as T2IN (that is default)

; Timer1 management
;T1GCON_INI	equ 0x00	; T1GE disabled; T1GTM disabled; T1GPOL low; T1GGO done; T1GSPM disabled; 
;T1GATE_INI	equ 0x00	; GSS T1G_pin; 
;T1CLK_INI	equ 0x01	; ClockSelect FOSC/4; 
;;TMR1H = 0x00;    //TMR1H 0; 
;;TMR1L = 0x00;    //TMR1L 0; 
;T1CON_INI	equ 0x14	; CKPS 1:2; nT1SYNC do_not_synchronize; TMR1ON disabled; T1RD16 disabled; 

; Timer0 management		used for read keyboard
T0CON1_INI	equ b'01010101'	; T0CS Fosc/4; T0CKPS 1:32; T0ASYNC not_synchronised; 
T0CON0_INI	equ b'10000000'	; T0OUTPS 1:1; T0EN enabled; T016BIT 8-bit; 
TMR0H_INI	equ .155		; in 8 bit mode the TMR0H is compared same as PR2 with timer2 --> 250 000 / (32 * 156) = 50.08 Hz 
;TMR0L = 0x00; TMR0L 0; 

; EUSART management
;BAUD1CON_INI	equ 0x08	; ABDOVF no_overflow; SCKP Non-Inverted; BRG16 16bit_generator; WUE disabled; ABDEN disabled; 
;RC1STA_INI	equ 0x90	; SPEN enabled; RX9 8-bit; CREN enabled; ADDEN disabled; SREN disabled; 
;TX1STA_INI	equ 0x04	; TX9 8-bit; TX9D 0; SENDB sync_break_complete; TXEN disabled; SYNC asynchronous; BRGH hi_speed; CSRC slave; 
;SP1BRGL_INI	equ 0x1D	; SP1BRGL 29; 
;SP1BRGL_INI	equ 0x77	; SP1BRGL 119; 
;SP1BRGH_INI	equ 0x00	; SP1BRGH 0; 

; CLC 1 management
;CLC1CON_INI	equ	b'10000010'	; LC1EN is enabled, LC1INT disabled, LC1MODE is 4 input AND
;CLC1POL_INI	equ	b'00001100'	; LC1G1 not inverted, LC1G2 not inverted, LC1G3 inverted, LC1G4 inverted, LC1POL not invrted
;CLC1SEL0_INI	equ	b'00000000'	; data 0 is CLCIN0PPS
;CLC1SEL1_INI	equ	b'00000100'	; data 1 is Fosc
;CLC1SEL2_INI	equ	b'00000000'	; data 2 is CLCIN0PPS
;CLC1SEL3_INI	equ	b'00000000'	; data 3 is CLCIN0PPS
;CLC1GLS0_INI	equ	b'00000010'	; gate 0 input is data 0 non inverted
;CLC1GLS1_INI	equ	b'00001000'	; gate 0 input is data 1 non inverted
;CLC1GLS2_INI	equ	b'00000000'	; gate 2 input is none
;CLC1GLS3_INI	equ	b'00000000'	; gate 3 input is none

;INTPPS_INI		equ	b'00000010'	; map INT to RA2
;CLCIN0PPS_INI	equ	b'00000010'	; map CLCIN0 to RA2

; Interrupt section
PIE0_INI	equ 0x00	; none used
PIE1_INI	equ 0x00	; none used
PIE2_INI	equ 0x00	; none used
PIE3_INI	equ 0x00	; none used
PIE4_INI	equ 0x00	; none used
PIE5_INI	equ 0x00	; none used
PIE6_INI	equ 0x00	; none used
PIE7_INI	equ 0x00	; none used

INTC_INI	equ 0x00	; no interrupts; all in main


#define		SER595	LATA,2			; serial data to 595
#define		SCK595	LATA,1			; serial clock for 595
#define		RCK595	LATA,0			; parallel load to 595

#define		KEYPORT	PORTC			; Port with keyboard


; --- EEPROM Section - no EEPROM, it is emulated by SAF
#define		SAF1_INI	0x00780

; ----- Variables

; --- Internal RAM Section
		cblock  RAMINT
;NEWTRISA	; new state for TRISA  \
;NEWLATA		; new state for LATA   \\
;NEWTRISC	; new state for TRISC  -- charlieplexing registers.
;NEWLATC		; new state for LATC   _/
;PORTTMP		; port input copy - used for read button
		endc

;#define		SWIN2	PORTTMP,3			; reed switch pin


		cblock  RAMINI0

C_SEC		; counter of seconds
C_MIN		; counter of minutes
C_HOURS		; counter of hours
BIT_CNT		; Counter of bits (pixels)

; 12:34
;    a
;   ---
; f| g |b
;   ---
; e|   |c
;   ---
;    d
; dots between hours and mins are segment f from digit 1

TEMP		; various temps :)
TEMP2		; various temps :)
COUNT

BTNS		; button matrix
OLDBTNS		; copy of button matrix used for momentary buttons

FLAGS		; status flags

       endc

; --- Flags
						; FLAGS
#define		SIX_DIGITS	FLAGS,0		; mode is changed to 6 digit display 12:34:56 (default is 4 digit 12:34)

#define		BTN_SS	BTNS,0		; Button for show seconds
#define		BTN_S0	BTNS,1		; Button for set seconds to 0
#define		BTN_MM	2		; Button for Minute Minus
#define		BTN_MP	3		; Button for Minute Plus
#define		BTN_HM	4		; Button for Hour Minus
#define		BTN_HP	5		; Button for Hour Plus


; ------ Program Section

		org	0x000

PowerUp:
		;clrf	STATUS			; ensure we are at bank 0
		clrf	PCLATH			; ensure page bits before goto !!
		clrf	INTCON			; disable all interrupts
		goto	Start


;
;**********************************************************************************************************************
; ISR (Interrupt Service Routines)
;**********************************************************************************************************************
;
	org	0x004
Interrupt:
	; no interrupt
	retfie


; 12:34
;    a
;   ---
; f| g |b
;   ---
; e|   |c
;   ---
;    d
; dots between hours and mins are segment f from digit 1

CharSet:		; 7 segment character set
		brw
		;        gfedcba
		retlw b'00111111'	; 0
		retlw b'00000110'	; 1
		retlw b'01011011'	; 2
		retlw b'01001111'	; 3
		retlw b'01100110'	; 4
		retlw b'01101101'	; 5
		retlw b'01111101'	; 6
		retlw b'00000111'	; 7
		retlw b'01111111'	; 8
		retlw b'01101111'	; 9
		retlw b'00011110'	; J
		retlw b'01110001'	; F
		
		;              gfedcba
#define		CHARJ	b'00011110'		; character J
#define		CHARI	b'00000100'		; character i
#define		CHARN	b'01010100'		; character n
#define		CHARD	b'01011110'		; character d
#define		CHARR	b'01010000'		; character r
#define		CHARA	b'01110111'		; character a
#define		CHARF	b'01110001'		; character F
#define		CHARU	b'00011100'		; character u


Start:
	BANKSEL	PORTA		; BANK 0
	movlw	LATA_INI	; all outputs to zero
	movwf	LATA
	movlw	TRISA_INI	; RA2 is output, RA3 is MCLR
	movwf	TRISA
	movlw	LATC_INI	; all outputs to zero
	movwf	LATC
	movlw	TRISC_INI	; RA2 is output, RA3 is MCLR
	movwf	TRISC

	; PMD = PERIPHERAL MODULE DISABLE
	BANKSEL	PMD0		; BANK 15
	movlw	PMD0_INI	; CLKRMD CLKR enabled; SYSCMD SYSCLK enabled; FVRMD FVR disabled; IOCMD IOC enabled; NVMMD NVM enabled; 
	movwf	PMD0
	movlw	PMD1_INI	; TMR0MD TMR0 enabled; TMR1MD TMR1 enabled; TMR2MD TMR2 enabled; NCOMD DDS(NCO) enabled; 
	movwf	PMD1
	movlw	PMD2_INI	; ZCDMD ZCD disabled; CMP1MD CMP1 enabled; ADCMD ADC disabled; CMP2MD CMP2 disabled; DAC1MD DAC1 disabled; 
	movwf	PMD2
	movlw	PMD3_INI	; CCP2MD CCP2 disabled; CCP1MD CCP1 disabled; PWM4MD PWM4 disabled; PWM3MD PWM3 disabled; PWM6MD PWM6 disabled; PWM5MD PWM5 disabled; 
	movwf	PMD3
	movlw	PMD4_INI	; CWG1MD CWG1 disabled; MSSP1MD MSSP1 disabled; UART1MD EUSART enabled; 
	movwf	PMD4
	movlw	PMD5_INI	; CLC3MD CLC3 disabled; CLC4MD CLC4 disabled; CLC1MD CLC1 disabled; CLC2MD CLC2 disabled; 
	movwf	PMD5

; PIN manager
	BANKSEL	ANSELA		; BANK 62

	movlw	ANSELA_INI	; 
	movwf	ANSELA
	movlw	ANSELC_INI	; 
	movwf	ANSELC
	movlw	WPUA_INI	; 
	movwf	WPUA
	movlw	WPUC_INI	; 
	movwf	WPUC
	movlw	ODCONA_INI	; disable open drain outputs
	movwf	ODCONA
	movlw	ODCONC_INI	; disable open drain outputs
	movwf	ODCONC

; oscilator management
	BANKSEL	OSCCON1		; BANK 17
	movlw	OSCCON1_INI	; NOSC HFINTOSC; NDIV 4; 
	movwf	OSCCON1
		;OSCCON2 is read only
	movlw	OSCCON3_INI	; CSWHOLD may proceed; 
	movwf	OSCCON3
	movlw	OSCEN_INI	; MFOEN disabled; LFOEN disabled; ADOEN disabled; EXTOEN disabled; HFOEN disabled; 
	movwf	OSCEN
	movlw	OSCFRQ_INI	; HFFRQ 8_MHz; 
	movwf	OSCFRQ
	movlw	OSCSTAT_INI	; MFOR not ready; do not understand, it is read only
	movwf	OSCSTAT
	movlw	OSCTUNE_INI	; HFTUN 32; do not understand why MCC set this?
	movwf	OSCTUNE

; Timer2 management
	BANKSEL	TMR2		; BANK 5
	movlw	T2CLKCON_INI	; T2CS FOSC/4; 
	movwf	T2CLKCON
	movlw	T2HLT_INI	; T2PSYNC Not Synchronized; T2MODE Software control; T2CKPOL Rising Edge; T2CKSYNC Not Synchronized; 
	movwf	T2HLT
	movlw	T2RST_INI	; T2RSEL T2INPPS pin; 
	movwf	T2RST
	movlw	T2PR_INI	; T2PR 59 (0.3B); 
	movwf	T2PR
;		;T2TMR = 0x00; TMR2 0; 
	movlw	T2CON_INI	; T2CKPS 1:2; T2OUTPS 1:1; TMR2ON on; 
	movwf	T2CON

; Timer0 management
	BANKSEL	T0CON1		; BANK 11
	movlw	T0CON1_INI	; T0CS Fosc/4; T0CKPS 1:2; T0ASYNC not_synchronised; 
	movwf	T0CON1
	movlw	T0CON0_INI	; T0OUTPS 1:1; T0EN enabled; T016BIT 8-bit; 
	movwf	T0CON0
	movlw	TMR0H_INI	; in 8 bit mode the TMR0H is compared same as PR2 with timer2 --> 500 000 / (2 * 178) = 1404.5 Hz --> 28 segments --> 1404.5 / 28 = 50.16 Hz per screen
	movwf	TMR0H


; Interrupts
	BANKSEL	PIE0	; BANK 2
	movlw	PIE0_INI	; none used
	movwf	PIE0
	movlw	PIE1_INI	; none used
	movwf	PIE1
	movlw	PIE2_INI	; none used
	movwf	PIE2
	movlw	PIE3_INI	; none used
	movwf	PIE3
	movlw	PIE4_INI	; none used
	movwf	PIE4
	movlw	PIE5_INI	; none used
	movwf	PIE5
	movlw	PIE6_INI	; none used
	movwf	PIE6
	movlw	PIE7_INI	; none used
	movwf	PIE7

	movlb	0	; BANK 0
	movlw	RAMINI0			; Clear variables bank0
	movwf	FSR0L
	clrf	FSR0H
	clrw
ClearRAM:
	movwi	FSR0++
	;clrf	INDF0
	;incf	FSR0L,f
	btfss	FSR0L,7			; to address 7F
	goto	ClearRAM

	movlb	0	; BANK 0
	movlw	INTC_INI		; Set interrupts (no interrupts)
	movwf	INTCON

	movf	KEYPORT,w	; load keyboard status
	andlw	0x3F
	movwf	BTNS		; save it to variable
	movwf	OLDBTNS		; ... and old variable

	bsf		SIX_DIGITS
	btfss	BTN_SS
	goto	Logo6digit

	bcf		SIX_DIGITS

	movlw CHARJ	; J		4 digit display "Ji Fu"
	call UploadDigit
	movlw CHARI	; i
	call UploadDigit
	clrw		; dots
	call UploadDigit
	movlw CHARF	; F
	call UploadDigit
	movlw CHARU	; u
	call UploadDigit
	call SubmitScreen
	goto MainLoop

Logo6digit:
	call	DrawScreen6DigSS	; this one display the logo

MainLoop:
	BANKSEL	PIR4		; BNK 14
	btfsc	PIR4,TMR2IF	; next second
	call	IncSec		; increase

	BANKSEL	PIR0		; BNK 14
	btfsc	PIR0,TMR0IF	; time for check keyb
	call	CheckKeyb	; prepare next segment for interrupt

	goto	MainLoop

IncSec:
	bcf		PIR4,TMR2IF	; clear flag
	movlb	0
	incf	C_SEC,f	; plus one sec
	movf	C_SEC,w	; is it 60? then must be set to 00
	xorlw	.60
	;xorlw	.10
	btfss	STATUS,Z
	goto	DrawScreen	; some change on digits appear, screen should be re-draw

	clrf	C_SEC		; set 00
	incf	C_MIN,f	; plus one min
	movf	C_MIN,w	; is it 60? then must be set to 00
	xorlw	.60
	btfss	STATUS,Z
	goto	DrawScreen	; some change on digits appear, screen should be re-draw

	clrf	C_MIN		; set 00
	incf	C_HOURS,f	; plus one hour
	movf	C_HOURS,w	; is it 24? then must be set to 00
	xorlw	.24
	btfss	STATUS,Z
	goto	DrawScreen	; some change on digits appear, screen should be re-draw
	clrf	C_HOURS		; set 00

DrawScreen:				; time to refresh screen
	btfsc	SIX_DIGITS	; for 6 digit display we have different view
	goto	DrawScreen6Dig
	btfss	BTN_SS		; display second or hours:minutes??
	goto	DrawScreenSS
	movf	C_HOURS,w	; get hours
	call	BinaryToBcd	; convert to BCD
	movwf	TEMP		; save for 2nd digit
	swapf	TEMP,w		; upper digit
	andlw	0x0F		; upper digit only
	btfss	STATUS,Z	; do not display leading zeroes
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen
	movf	TEMP,w		; lower digit
	andlw	0x0F		; lower digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen

	movlw	0x03		; two dots blinking by seconds
	btfss	C_SEC,0		; 
	clrw				; not lighting
	call	UploadDigit	; put to screen

	movf	C_MIN,w		; get minutes
	call	BinaryToBcd	; convert to BCD
	movwf	TEMP		; save for 2nd digit
	swapf	TEMP,w		; upper digit
	andlw	0x0F		; upper digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen
	movf	TEMP,w		; lower digit
	andlw	0x0F		; lower digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen

	;return
	goto	SubmitScreen

DrawScreenSS:
	movlw CHARJ	; J
	call	UploadDigit	; put to screen
	movlw CHARF	; F
	call	UploadDigit	; put to screen

	movlw	0x03		; two dots blinking by seconds
	btfss	C_SEC,0		; 
	clrw				; not lighting
	call	UploadDigit	; put to screen

	movf	C_SEC,w		; get seconds
	call	BinaryToBcd	; convert to BCD
	movwf	TEMP		; save for 2nd digit
	swapf	TEMP,w		; upper digit
	andlw	0x0F		; upper digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen
	movf	TEMP,w		; lower digit
	andlw	0x0F		; lower digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen

	;return
	goto	SubmitScreen

DrawScreen6Dig
	btfss	BTN_SS		; display second or hours:minutes??
	goto	DrawScreen6DigSS
	movf	C_HOURS,w	; get hours
	call	BinaryToBcd	; convert to BCD
	movwf	TEMP		; save for 2nd digit
	swapf	TEMP,w		; upper digit
	andlw	0x0F		; upper digit only
	;btfss	STATUS,Z	; do not display leading zeroes
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen
	movf	TEMP,w		; lower digit
	andlw	0x0F		; lower digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen

	movlw	0x03		; two dots blinking by seconds
	btfss	C_SEC,0		; 
	clrw				; not lighting
	call	UploadDigit	; put to screen

	movf	C_MIN,w		; get minutes
	call	BinaryToBcd	; convert to BCD
	movwf	TEMP		; save for 2nd digit
	swapf	TEMP,w		; upper digit
	andlw	0x0F		; upper digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen
	movf	TEMP,w		; lower digit
	andlw	0x0F		; lower digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen

	movlw	0x03		; two dots blinking by seconds
	btfsc	C_SEC,0		; 
	clrw				; not lighting
	call	UploadDigit	; put to screen

	movf	C_SEC,w		; get seconds
	call	BinaryToBcd	; convert to BCD
	movwf	TEMP		; save for 2nd digit
	swapf	TEMP,w		; upper digit
	andlw	0x0F		; upper digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen
	movf	TEMP,w		; lower digit
	andlw	0x0F		; lower digit only
	call	CharSet		; convert to 7 segments
	call	UploadDigit	; put to screen

	;return
	goto	SubmitScreen

DrawScreen6DigSS:
	movlw CHARJ	; J		6 digit display "Ji nd ra"
	call UploadDigit
	movlw CHARI	; i
	call UploadDigit
	clrw		; dots
	call UploadDigit
	movlw CHARN	; n
	call UploadDigit
	movlw CHARD	; d
	call UploadDigit
	clrw		; dots
	call UploadDigit
	movlw CHARR	; r
	call UploadDigit
	movlw CHARA	; a
	call UploadDigit
	;return
	goto	SubmitScreen


UploadDigit:
	movwf TEMP2

	movlw	0x08
	movwf	COUNT
NextDigitBites:
	bcf	SER595
	btfsc	TEMP2,7
	bsf	SER595
	rlf	TEMP2,f			; next bite
	DNOP
	bsf	SCK595			; clock _/-
	DNOP
	DNOP
	bcf	SCK595		; clock -\_
	DNOP
	decfsz	COUNT,f
	goto	NextDigitBites
	return

SubmitScreen:
	DNOP
	bsf	RCK595		; parallel write _/-
	DNOP
	DNOP
	bcf	RCK595		; parallel write -\_
	return

;******************************** 
;binary_to_bcd - 8-bits
;
;Input
;  8-bit binary number
;   A1*16+A0
;Outputs
; tens_and_ones - the tens and ones digits of the BCD conversion
BinaryToBcd:

	movwf	TEMP
	SWAPF	TEMP, W		; swap the nibbles
	ADDWF	TEMP, W		; so we can add the upper to the lower
	ANDLW	B'00001111'	; lose the upper nibble (W is in BCD from now on)
	BTFSC	STATUS, DC	; if we carried a one (upper + lower > 16)
	 ADDLW	0x16		; add 16 (the place value) (1s + 16 * 10s)
	BTFSC	STATUS, DC	; did that cause a carry from the 1's place?
	 ADDLW	0x06		; if so, add the missing 6 (carry is only worth 10)
	ADDLW	0x06        ; fix max digit value by adding 6
	BTFSS	STATUS, DC	; if was greater than 9, DC will be set
	 ADDLW	-0x06		; if if it wasn't, get rid of that extra 6

	BTFSC	TEMP,4		; 16's place
	 ADDLW	0x16 - 1 + 0x6	; add 16 - 1 and check for digit carry
	BTFSS	STATUS, DC
	 ADDLW	-0x06       ; if nothing carried, get rid of that 6

	BTFSC	TEMP, 5      ; 32nd's place
	 ADDLW	0x30        ; add 32 - 2

	BTFSC	TEMP, 6      ; 64th's place		Time is always < 64 :)
	 ADDLW	0x60        ; add 64 - 4

	BTFSC	TEMP, 7      ; 128th's place	Time is always < 128 :)
	 ADDLW	0x20        ; add 128 - 8 % 100

	RETURN              ; all done!

CheckKeyb:	; cca 50 Hz for keyboard
	bcf	PIR0,TMR0IF	; time for check keyb
	;return

	movlb	0
	movf	KEYPORT,w	; load keyboard status
	andlw	0x3F
	movwf	BTNS		; save it to variable
	btfss	BTN_S0		; set seconds to zero
	clrf	C_SEC		; done	possible future improvement is to clear Timer2
	movf	BTNS,w		; button matrix
	xorwf	OLDBTNS,w	; copy of button matrix used for momentary buttons
	btfsc	STATUS,Z	; check for momentary buttons
	return				; no change
	movwf	TEMP		; save changes

	btfsc	BTNS,BTN_HM
	btfss	TEMP,BTN_HM
	goto	SkipBtnHM	; no Hour Minus button pressed
	decf	C_HOURS,f	; minus one hour
	incf	C_HOURS,w	; was it 00? then must be set to 23
	btfss	STATUS,Z
	goto	SkipBtnHM	; no, can continue
	movlw	.23
	movwf	C_HOURS

SkipBtnHM:
	btfsc	BTNS,BTN_HP
	btfss	TEMP,BTN_HP
	goto	SkipBtnHP	; no Hour Plus button pressed
	incf	C_HOURS,f	; plus one hour
	movf	C_HOURS,w	; is it 24? then must be set to 00
	xorlw	.24
	btfsc	STATUS,Z
	clrf	C_HOURS		; set 00

SkipBtnHP:
	btfsc	BTNS,BTN_MM
	btfss	TEMP,BTN_MM
	goto	SkipBtnMM	; no Minute Minus button pressed
	decf	C_MIN,f	; minus one minute
	incf	C_MIN,w	; was it 00? then must be set to 59
	btfss	STATUS,Z
	goto	SkipBtnMM	; no, can continue
	movlw	.59
	movwf	C_MIN

SkipBtnMM:
	btfsc	BTNS,BTN_MP
	btfss	TEMP,BTN_MP
	goto	SkipBtnMP	; no Minute Plus button pressed
	incf	C_MIN,f	; plus one hour
	movf	C_MIN,w	; is it 60? then must be set to 00
	xorlw	.60
	btfsc	STATUS,Z
	clrf	C_MIN		; set 00

SkipBtnMP:
	movf	BTNS,w		; copy buttons to old buttons
	movwf	OLDBTNS
	goto	DrawScreen	; some change on digits appear, screen should be re-draw

; ----------------------------------------------------------------------

; ----------------------------------------------------------------------
	org	SAF1_INI

		dt	"Filament"
		dt	" J.Fucik"
		dt	(__VERDAY   >> 4)  +0x30
		dt	(__VERDAY   & 0x0F)+0x30,"/"
		dt	(__VERMONTH >> 4)  +0x30
		dt	(__VERMONTH & 0x0F)+0x30,"/"
		dt	(__VERYEAR  >> 4)  +0x30
		dt	(__VERYEAR  & 0x0F)+0x30

		end